@@ -1,5 +1,3 @@
-package org.debian.maven.repo;
-
 /*
  * Copyright 2009 Ludovic Claude.
  *
@@ -16,14 +14,28 @@ package org.debian.maven.repo;
  * limitations under the License.
  */
 
-import java.util.*;
+package org.debian.maven.repo;
+
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
+import java.util.Set;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
 
 /**
  * Represents a Maven dependency of any type (dependency, plugin, parent POM)
  * 
  * @author Ludovic Claude <ludovicc@users.sourceforge.net>
  */
-public class Dependency implements Comparable, Cloneable {
+public class Dependency implements Comparable<Dependency>, Cloneable {
+
+    public static final Dependency PROTO_JAR = new Dependency(null, null, "jar", null);
+    public static final Dependency PROTO_PLUGIN = new Dependency("org.apache.maven.plugins", null, "maven-plugin", null);
+    private static final Pattern COMPACT_DEPENDENCY_NOTATION_MATCHER =
+            Pattern.compile("(\\w[a-zA-Z0-9\\-_\\.]*):(\\w[a-zA-Z0-9\\-_]*):(\\d[a-zA-Z0-9\\-_\\.]*)");
 
     private String groupId;
     private String artifactId;
@@ -67,10 +79,23 @@ public class Dependency implements Comparable, Cloneable {
         this.superPom = dependency.superPom;
     }
 
+    public Dependency(Builder builder) {
+        this.groupId = builder.groupId;
+        this.artifactId = builder.artifactId;
+        this.type = builder.type;
+        this.version = builder.version;
+        this.scope = builder.scope;
+        this.optional = builder.optional;
+        this.classifier = builder.classifier;
+        this.relativePath = builder.relativePath;
+        this.superPom = builder.superPom;
+    }
+
     public String getArtifactId() {
         return artifactId;
     }
 
+    @Deprecated
     public void setArtifactId(String artifactId) {
         this.artifactId = artifactId;
     }
@@ -79,6 +104,7 @@ public class Dependency implements Comparable, Cloneable {
         return groupId;
     }
 
+    @Deprecated
     public void setGroupId(String groupId) {
         this.groupId = groupId;
     }
@@ -87,6 +113,7 @@ public class Dependency implements Comparable, Cloneable {
         return type;
     }
 
+    @Deprecated
     public void setType(String type) {
         this.type = type;
     }
@@ -95,6 +122,7 @@ public class Dependency implements Comparable, Cloneable {
         return version;
     }
 
+    @Deprecated
     public void setVersion(String version) {
         this.version = version;
     }
@@ -103,6 +131,7 @@ public class Dependency implements Comparable, Cloneable {
         return optional;
     }
 
+    @Deprecated
     public void setOptional(boolean optional) {
         this.optional = optional;
     }
@@ -115,6 +144,7 @@ public class Dependency implements Comparable, Cloneable {
         return scope == null ? "runtime" : scope;
     }
 
+    @Deprecated
     public void setScope(String scope) {
         this.scope = scope;
     }
@@ -123,6 +153,7 @@ public class Dependency implements Comparable, Cloneable {
         return classifier;
     }
 
+    @Deprecated
     public void setClassifier(String classifier) {
         this.classifier = (classifier == null) ? "" : classifier;
     }
@@ -131,10 +162,12 @@ public class Dependency implements Comparable, Cloneable {
         return relativePath;
     }
 
+    @Deprecated
     public void setRelativePath(String relativePath) {
         this.relativePath = relativePath;
     }
 
+    @Deprecated
     public void setSuperPom(boolean superPom) {
         this.superPom = superPom;
     }
@@ -143,6 +176,34 @@ public class Dependency implements Comparable, Cloneable {
         return superPom;
     }
 
+    /**
+     * Tells if this artifact is a pom.
+     */
+    public boolean isPom() {
+        return "pom".equals(type);
+    }
+
+    /**
+     * Tells if this artifact is a jar.
+     */
+    public boolean isJar() {
+        return "jar".equals(type);
+    }
+
+    /**
+     * Tells if this artifact is a Maven plugin.
+     */
+    public boolean isPlugin() {
+        return "maven-plugin".equals(type);
+    }
+
+    /**
+     * Tells if this artifact is an OSGi bundle.
+     */
+    public boolean isBundle() {
+        return "bundle".equals(type);
+    }
+
     public boolean equals(Object obj) {
         if (obj == null) {
             return false;
@@ -200,8 +261,7 @@ public class Dependency implements Comparable, Cloneable {
         return groupId + ":" + artifactId + ":" + type + ":" + version;
     }
 
-    public int compareTo(Object o) {
-        Dependency dependency = (Dependency) o; 
+    public int compareTo(Dependency dependency) {
         if (this.groupId != dependency.groupId) {
             if (this.groupId == null) return -1;
             if (! this.groupId.equals(dependency.groupId)) {
@@ -285,4 +345,99 @@ public class Dependency implements Comparable, Cloneable {
         return type2 == null;
     }
 
+    public static Dependency fromCompactNotation(String depNotation) {
+        Matcher dependencyMatcher = COMPACT_DEPENDENCY_NOTATION_MATCHER.matcher(depNotation);
+        if (dependencyMatcher.matches()) {
+            return new Dependency(dependencyMatcher.group(1),
+                    dependencyMatcher.group(2), "jar", dependencyMatcher.group(3));
+        }
+        return null;
+    }
+
+    public String formatCompactNotation() {
+        return getGroupId() + ":" + getArtifactId() + ":" + getVersion();
+    }
+
+    public Builder builder() {
+        Builder builder = new Builder();
+        builder.groupId = groupId;
+        builder.artifactId = artifactId;
+        builder.type = type;
+        builder.version = version;
+        builder.optional = optional;
+        builder.scope = scope;
+        builder.classifier = classifier;
+        builder.relativePath = relativePath;
+        builder.superPom = superPom;
+        return builder;
+    }
+
+    public static class Builder {
+        private String groupId;
+        private String artifactId;
+        private String type;
+        private String version;
+        private boolean optional;
+        private String scope;
+        private String classifier;
+        private String relativePath;
+        private boolean superPom;
+
+        public Builder setGroupId(String groupId) { this.groupId = groupId; return this; }
+        public Builder setArtifactId(String artifactId) { this.artifactId = artifactId; return this; }
+        public Builder setType(String type) { this.type = type; return this; }
+        public Builder setVersion(String version) { this.version = version; return this; }
+        public Builder setOptional(boolean optional) { this.optional = optional; return this; }
+        public Builder setScope(String scope) { this.scope = scope; return this; }
+        public Builder setClassifier(String classifier) { this.classifier = classifier; return this; }
+        public Builder setRelativePath(String relativePath) { this.relativePath = relativePath; return this; }
+        public Builder setSuperPom(boolean superPom) { this.superPom = superPom; return this; }
+
+        public Builder set(Field field, String value) {
+            switch (field) {
+                case GROUPID: setGroupId(value); return this;
+                case ARTIFACTID: setArtifactId(value); return this;
+                case CLASSIFIER: setClassifier(value); return this;
+                case OPTIONAL: setOptional("true".equals(value)); return this;
+                case RELATIVEPATH: setRelativePath(value); return this;
+                case SCOPE: setScope(value); return this;
+                case TYPE: setType(value); return this;
+                case VERSION: setVersion(value); return this;
+            }
+            return this;
+        }
+
+        public Dependency build() {
+            return new Dependency(this);
+        }
+    }
+
+    public enum Field {
+        GROUPID("groupId"),
+        ARTIFACTID("artifactId"),
+        TYPE("type"),
+        VERSION("version"),
+        OPTIONAL("optional"),
+        SCOPE("scope"),
+        CLASSIFIER("classifier"),
+        RELATIVEPATH("relativePath");
+
+        public final static Map<String, Field> map;
+        static {
+            map = new HashMap<String, Field>();
+            for(Field field : Field.values()) {
+                map.put(field.TAG, field);
+            }
+        }
+
+        public final String TAG;
+
+        Field(String tag) {
+            TAG = tag;
+        }
+
+        public static Field get(String tag) {
+            return map.get(tag);
+        }
+    }
 }
