@@ -1,5 +1,3 @@
-package org.debian.maven.repo;
-
 /*
  * Copyright 2009 Ludovic Claude.
  *
@@ -16,59 +14,70 @@ package org.debian.maven.repo;
  * limitations under the License.
  */
 
+package org.debian.maven.repo;
+
 import java.util.StringTokenizer;
 
 /**
  *
  * @author Ludovic Claude <ludovicc@users.sourceforge.net>
  */
-public class DependencyRule implements Comparable {
+public class DependencyRule implements Comparable<DependencyRule> {
 
-    public static DependencyRule TO_DEBIAN_VERSION_RULE = new DependencyRule("");
-    public static DependencyRule MAVEN_PLUGINS_KEEP_VERSION_RULE = new DependencyRule("* * maven-plugin * * *");
-    public static DependencyRule NO_CHANGE_RULE = new DependencyRule("* * * * * *");
+    private static final Rule STAR_RULE = new Rule("*");
+    private static final Rule DEBIAN_RULE = new Rule("s/.*/debian/");
 
-    private Rule groupRule;
-    private Rule artifactRule;
-    private Rule typeRule;
-    private Rule versionRule;
-    private Rule scopeRule;
-    private Rule classifierRule;
+    private final Rule groupRule;
+    private final Rule artifactRule;
+    private final Rule typeRule;
+    private final Rule versionRule;
+    private final Rule scopeRule;
+    private final Rule classifierRule;
 
     public DependencyRule(String def) {
         StringTokenizer st = new StringTokenizer(def, " \t", false);
+        groupRule = maybeParseRule(st);
+        artifactRule = maybeParseRule(st);
+        typeRule = maybeParseRule(st);
+        versionRule = maybeParseRule(st, DEBIAN_RULE);
+        classifierRule = maybeParseRule(st);
+        scopeRule = maybeParseRule(st);
+    }
+
+    public DependencyRule(String group, String artifact, String type, String version, String classifier, String scope) {
+        groupRule = new Rule(group);
+        artifactRule = new Rule(artifact);
+        typeRule = new Rule(type);
+        versionRule = new Rule(version);
+        classifierRule = new Rule(classifier);
+        scopeRule = new Rule(scope);
+    }
+
+    public DependencyRule(String group, String artifact, String type, String version) {
+        this(group, artifact, type, version, STAR_RULE.toString(), STAR_RULE.toString());
+    }
+
+    public static DependencyRule newToMatch(Dependency dependency) {
+        return new DependencyRule(
+            dependency.getGroupId(),
+            dependency.getArtifactId(),
+            dependency.getType() == null ? "*" : dependency.getType(),
+            "*"
+            );
+    }
+
+    private static Rule maybeParseRule(StringTokenizer st, Rule defaultRule) {
         if (st.hasMoreTokens()) {
-            groupRule = new Rule(st.nextToken());
+            return new Rule(st.nextToken());
         } else {
-            groupRule = new Rule("*");
-        }
-        if (st.hasMoreTokens()) {
-            artifactRule = new Rule(st.nextToken());
-        } else {
-            artifactRule = new Rule("*");
-        }
-        if (st.hasMoreTokens()) {
-            typeRule = new Rule(st.nextToken());
-        } else {
-            typeRule = new Rule("*");
-        }
-        if (st.hasMoreTokens()) {
-            versionRule = new Rule(st.nextToken());
-        } else {
-            versionRule = new Rule("s/.*/debian/");
-        }
-        if (st.hasMoreTokens()) {
-            classifierRule = new Rule(st.nextToken());
-        } else {
-            classifierRule = new Rule("*");
-        }
-        if (st.hasMoreTokens()) {
-            scopeRule = new Rule(st.nextToken());
-        } else {
-            scopeRule = new Rule("*");
+            return defaultRule;
         }
     }
 
+    private static Rule maybeParseRule(StringTokenizer st) {
+        return maybeParseRule(st, STAR_RULE);
+    }
+
     public Rule getArtifactRule() {
         return artifactRule;
     }
@@ -94,8 +103,11 @@ public class DependencyRule implements Comparable {
     }
 
     public boolean matches(Dependency dependency) {
-        return groupRule.match(dependency.getGroupId()) && artifactRule.match(dependency.getArtifactId()) && typeRule.match(dependency.getType()) 
-                && versionRule.match(dependency.getVersion()) && scopeRule.match(dependency.getScope())
+        return groupRule.match(dependency.getGroupId())
+                && artifactRule.match(dependency.getArtifactId())
+                && typeRule.match(dependency.getType()) 
+                && versionRule.match(dependency.getVersion())
+                && scopeRule.match(dependency.getScope())
                 && classifierRule.match(dependency.getClassifier());
     }
 
@@ -110,15 +122,30 @@ public class DependencyRule implements Comparable {
                 dependency.getRelativePath());
     }
 
+    public boolean matchesAndPreservesGroupArtifactAndType(Dependency dependency) {
+        if (!matches(dependency)) {
+            return false;
+        }
+        Dependency transformed = apply(dependency);
+        return transformed.getGroupId().equals(dependency.getGroupId())
+                && transformed.getArtifactId().equals(dependency.getArtifactId())
+                && transformed.getType().equals(dependency.getType());
+    }
+
+    public boolean explicitlyMentions(Dependency dependency) {
+        return !this.equals(DependencyRuleSet.TO_DEBIAN_VERSION_RULE)
+            && !this.equals(DependencyRuleSet.MAVEN_PLUGINS_KEEP_VERSION_RULE)
+            && this.matches(dependency);
+    }
+
     /**
      * Natural sort order: from the most specific rules to the most generic rules,
-     * a rule is more generic if a generic match appears on th egroup rule, then
+     * a rule is more generic if a generic match appears on the group rule, then
      * artifact rule, then type rule, then version rule.
-     * If 2 rules have the same order of genericity, then use an alphabetical
+     * If 2 rules have the same order of genericness, then use an alphabetical
      * sorting of the pattern strings.
      */
-    public int compareTo(Object o) {
-        DependencyRule other = (DependencyRule) o;
+    public int compareTo(DependencyRule other) {
         if (groupRule.isGeneric() && !other.groupRule.isGeneric()) {
             return 1;
         }
@@ -181,10 +208,7 @@ public class DependencyRule implements Comparable {
         if (this.classifierRule != other.classifierRule && (this.classifierRule == null || !this.classifierRule.equals(other.classifierRule))) {
             return false;
         }
-        if (this.scopeRule != other.scopeRule && (this.scopeRule == null || !this.scopeRule.equals(other.scopeRule))) {
-            return false;
-        }
-        return true;
+        return !(this.scopeRule != other.scopeRule && (this.scopeRule == null || !this.scopeRule.equals(other.scopeRule)));
     }
 
     public int hashCode() {
