@@ -1,5 +1,3 @@
-package org.debian.maven.repo;
-
 /*
  * Copyright 2009 Ludovic Claude.
  *
@@ -16,6 +14,8 @@ package org.debian.maven.repo;
  * limitations under the License.
  */
 
+package org.debian.maven.repo;
+
 import java.io.File;
 import java.io.FileOutputStream;
 import java.io.IOException;
@@ -23,7 +23,9 @@ import java.util.*;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 import javax.xml.stream.XMLStreamException;
-import javax.xml.stream.XMLStreamWriter;
+
+import org.debian.maven.repo.POMInfo.DependencyType;
+import static org.debian.maven.repo.DependencyRuleSetFiles.RulesType.*;
 
 /**
  * Cleans up a POM for inclusion in the /usr/share/maven-repo/ repository.
@@ -35,13 +37,12 @@ import javax.xml.stream.XMLStreamWriter;
 public class POMCleaner extends POMTransformer {
 
     private static final Logger log = Logger.getLogger(POMCleaner.class.getName());
-    private static final List WRITE_IGNORED_ELEMENTS = Arrays.asList(new String[]{"build",
+    private static final List<String> WRITE_IGNORED_ELEMENTS = Arrays.asList("build",
                 "distributionManagement", "profiles", "ciManagement", "prerequisites",
                 "repositories", "pluginRepositories", "reports", "reporting", "modelVersion",
-                "parent"});
+                "parent");
     private boolean keepAllElements = false;
-    private boolean isPOM = false;
-    private Collection keepElements = new ArrayList();
+    private Collection<String> keepElements = new ArrayList<String>();
 
     public POMCleaner() {
     }
@@ -85,25 +86,25 @@ public class POMCleaner extends POMTransformer {
 
         } catch (IOException ex) {
             log.log(Level.SEVERE, null, ex);
-            return;
         } catch (XMLStreamException ex) {
             log.log(Level.SEVERE, null, ex);
-            return;
         }
     }
 
     protected void transformingPom(POMInfo pom) {
-        if (pom.getThisPom().getType().equals("maven-plugin")) {
-            addIgnoreRule(new DependencyRule(pom.getThisPom().getGroupId() + " "
+        if (pom.getThisPom().isPlugin()) {
+            getRulesFiles().get(IGNORE).add(new DependencyRule(pom.getThisPom().getGroupId() + " "
                     + pom.getThisPom().getArtifactId() + " maven-plugin s/.*/"
                     + pom.getThisPom().getVersion() + "/"));
-        } else if (pom.getThisPom().getType().equals("pom")) {
-            isPOM = true;
         }
     }
 
-    protected boolean isWriteIgnoredElement(String element, List path, Dependency dependency) {
-        if ("relativePath".equals(element) && path.size() == 0) {
+    @Override
+    protected boolean shouldWriteRelativePath() { return false; }
+
+    @Override
+    protected boolean isWriteIgnoredElement(String element, TreePath<String> path, Dependency dependency) {
+        if (path.matches("parent/relativePath")) {
             return true;
         }
         boolean ignore = super.isWriteIgnoredElement(element, path, dependency);
@@ -115,8 +116,7 @@ public class POMCleaner extends POMTransformer {
         }
         if (path.size() > 2 && keepElements.contains(path.get(1))) {
             if ("version".equals(element)) {
-                String parent = (String) path.get(path.size() - 1);
-                if ("plugin".equals(parent)) {
+                if ("plugin".equals(path.parent(1))) {
                     return true;
                 }
             }
@@ -125,21 +125,17 @@ public class POMCleaner extends POMTransformer {
 //        if (WRITE_IGNORED_ELEMENTS.contains(element)) {
 //            System.out.println("Write ignored " + element + " " + printPath(path) + " for " + dependency);
 //        }
-        return path.size() == 1 && WRITE_IGNORED_ELEMENTS.contains(element);
+        return path.size() == 2 && WRITE_IGNORED_ELEMENTS.contains(element);
     }
 
     protected boolean acceptDependency(Dependency dependency, POMInfo info) {
-        if (!super.acceptDependency(dependency, info)) {
-            return false;
-        }
-        return "pom".equals(info.getThisPom().getType()) || !"test".equals(dependency.getScope());
+        return super.acceptDependency(dependency, info) &&
+                (info.getThisPom().isPom() || !"test".equals(dependency.getScope()));
     }
 
     protected void createDebianProperties(POMInfo info, POMInfo original, String debianPackage, int inLevel) throws XMLStreamException {
         super.createDebianProperties(info, original, debianPackage, inLevel);
-        Map dependencyVersions = new TreeMap();
-        for (Iterator i = original.getDependencies().iterator(); i.hasNext(); ) {
-            Dependency dependency = (Dependency) i.next();
+        for (Dependency dependency : original.getDependencies().get(DependencyType.DEPENDENCIES)) {
             if (dependency.getVersion() != null) {
                 String versionProperty = "debian." + dependency.getGroupId() + "." + dependency.getArtifactId() + ".originalVersion";
                 info.getProperties().put(versionProperty, dependency.getVersion());
@@ -271,10 +267,10 @@ public class POMCleaner extends POMTransformer {
         File rulesFile = null;
         File publishedRulesFile = null;
         File mavenRepo = null;
-        List rulesExtra = new ArrayList();
-        List publishedRulesExtra = new ArrayList();
-        List ignoreRulesExtra = new ArrayList();
-        List ignoreRulesFiles = new ArrayList();
+        List<String> rulesExtra = new ArrayList<String>();
+        List<String> publishedRulesExtra = new ArrayList<String>();
+        List<String> ignoreRulesExtra = new ArrayList<String>();
+        List<File> ignoreRulesFiles = new ArrayList<File>();
         while (i < args.length && (args[i].trim().startsWith("-") || args[i].trim().length() == 0)) {
             String arg = args[i].trim();
             if ("--verbose".equals(arg) || "-v".equals(arg)) {
@@ -350,43 +346,21 @@ public class POMCleaner extends POMTransformer {
         }
 
         if (!noRules) {
-            if (rulesFile != null) {
-                if (!rulesFile.exists()) {
-                    if (verbose) {
-                        System.err.println("Cannot find file: " + rulesFile);
-                    }
-                } else {
-                    cleaner.getRules().setRulesFile(rulesFile);
-                }
-            }
-            cleaner.getRules().addAll(rulesExtra);
+            DependencyRuleSet rules = cleaner.getRulesFiles().get(RULES);
+            rules.addAll(DependencyRuleSet.readRules(rulesFile, "", verbose, true));
+            rules.addAll(rulesExtra);
 
-            if (!ignoreRulesFiles.isEmpty()) {
-                boolean firstIgnoreRule = true;
-                for (Iterator irf = ignoreRulesFiles.iterator(); irf.hasNext(); ) {
-                    File ignoreRulesFile = (File) irf.next();
-                    if (ignoreRulesFile.exists()) {
-                        if (firstIgnoreRule) {
-                            cleaner.getIgnoreRules().setRulesFile(ignoreRulesFile);
-                        } else {
-                            DependencyRuleSet tmpIgnoreRules = new DependencyRuleSet("Additional ignore rules");
-                            tmpIgnoreRules.setRulesFile(ignoreRulesFile);
-                            cleaner.getIgnoreRules().addAll(tmpIgnoreRules);
-                        }
-                        firstIgnoreRule = false;
-                    } else {
-                        System.err.println("Cannot find file: " + ignoreRulesFile);
-                    }
-                }
+            DependencyRuleSet ignoreRules = cleaner.getRulesFiles().get(IGNORE);
+            for(File ignoreRulesFile : ignoreRulesFiles) {
+                ignoreRules.addAll(DependencyRuleSet.readRules(ignoreRulesFile, "", verbose, true));
             }
-            cleaner.getIgnoreRules().addAll(ignoreRulesExtra);
+            ignoreRules.addAll(ignoreRulesExtra);
 
-            if (publishedRulesFile != null && publishedRulesFile.exists()) {
-                cleaner.getPublishedRules().setRulesFile(publishedRulesFile);
-            }
-            cleaner.getPublishedRules().addAll(publishedRulesExtra);
+            DependencyRuleSet publishedRules = cleaner.getRulesFiles().get(PUBLISHED);
+            publishedRules.addAll(DependencyRuleSet.readRules(publishedRulesFile, "", verbose, true));
+            publishedRules.addAll(publishedRulesExtra);
 
-            cleaner.addDefaultRules();
+            cleaner.getRulesFiles().addDefaultRules();
         }
 
         if (mavenRepo != null) {
