@@ -1,7 +1,3 @@
-package org.debian.maven.repo;
-
-import java.util.Iterator;
-
 /*
  * Copyright 2009 Ludovic Claude.
  *
@@ -18,15 +14,28 @@ import java.util.Iterator;
  * limitations under the License.
  */
 
-public class POMReaderTest extends TestBase {
+package org.debian.maven.repo;
+
+import org.debian.maven.TemporaryPomFolder;
+import org.debian.maven.repo.POMReader.TreePath;
+import org.junit.Rule;
+import org.junit.Test;
+
+import static org.debian.maven.repo.POMInfo.DependencyType.*;
+import static org.junit.Assert.*;
+
+public class POMReaderTest {
+
+    @Rule
+    public TemporaryPomFolder tmpDir = new TemporaryPomFolder();
 
     /**
      * Test of cleanPom method, of class POMReader.
      */
+    @Test
     public void testReadMavenPom() throws Exception {
-        usePom("maven.xml");
         POMReader instance = new POMReader();
-        POMInfo info = instance.readPom(pom);
+        POMInfo info = instance.readPom(tmpDir.usePom("maven.xml"));
         assertNotNull(info.getParent());
         assertEquals("org.apache.maven", info.getParent().getGroupId());
         assertEquals("maven-parent", info.getParent().getArtifactId());
@@ -38,41 +47,41 @@ public class POMReaderTest extends TestBase {
         assertEquals("2.1.0-SNAPSHOT", info.getThisPom().getVersion());
         assertEquals("pom", info.getThisPom().getType());
 
-        assertEquals(1, info.getDependencies().size());
-        Dependency dependency = (Dependency) info.getDependencies().get(0);
+        assertEquals(1, info.getDependencies().get(DEPENDENCIES).size());
+        Dependency dependency = info.getDependencies().get(DEPENDENCIES).get(0);
         assertEquals("junit", dependency.getGroupId());
         assertEquals("junit", dependency.getArtifactId());
         assertEquals("3.8.1", dependency.getVersion());
         assertEquals("jar", dependency.getType());
 
-        assertEquals(34, info.getDependencyManagement().size());
-        dependency = (Dependency) info.getDependencyManagement().get(0);
+        assertEquals(34, info.getDependencies().get(DEPENDENCY_MANAGEMENT_LIST).size());
+        dependency = info.getDependencies().get(DEPENDENCY_MANAGEMENT_LIST).get(0);
         assertEquals("org.apache.maven", dependency.getGroupId());
         assertEquals("maven-plugin-descriptor", dependency.getArtifactId());
         assertEquals("2.1.0-SNAPSHOT", dependency.getVersion());
         assertEquals("jar", dependency.getType());
 
-        assertEquals(0, info.getProfileDependencies().size());
-        assertEquals(0, info.getProfileDependencyManagement().size());
+        assertEquals(0, info.getDependencies().get(PROFILE_DEPENDENCIES).size());
+        assertEquals(0, info.getDependencies().get(PROFILE_DEPENDENCY_MANAGEMENT_LIST).size());
 
-        assertEquals(14, info.getPluginManagement().size());
-        dependency = (Dependency) info.getPluginManagement().get(0);
+        assertEquals(14, info.getDependencies().get(PLUGIN_MANAGEMENT).size());
+        dependency = info.getDependencies().get(PLUGIN_MANAGEMENT).get(0);
         assertEquals("org.apache.maven.plugins", dependency.getGroupId());
         assertEquals("maven-jar-plugin", dependency.getArtifactId());
         assertEquals("2.1", dependency.getVersion());
         assertEquals("maven-plugin", dependency.getType());
 
-        assertEquals(0, info.getPlugins().size());
-        assertEquals(0, info.getReportingPlugins().size());
+        assertEquals(0, info.getDependencies().get(PLUGINS).size());
+        assertEquals(0, info.getDependencies().get(REPORTING_PLUGINS).size());
 
-        assertEquals(2, info.getProfilePlugins().size());
-        dependency = (Dependency) info.getProfilePlugins().get(0);
+        assertEquals(2, info.getDependencies().get(PROFILE_PLUGINS).size());
+        dependency = info.getDependencies().get(PROFILE_PLUGINS).get(0);
         assertEquals("org.codehaus.mojo", dependency.getGroupId());
         assertEquals("clirr-maven-plugin", dependency.getArtifactId());
         assertEquals(null, dependency.getVersion());
         assertEquals("maven-plugin", dependency.getType());
 
-        assertEquals(4, info.getProfileReportingPlugins().size());
+        assertEquals(4, info.getDependencies().get(PROFILE_REPORTING_PLUGINS).size());
 
         assertEquals(2, info.getProperties().size());
         assertEquals("2.1.0-SNAPSHOT", info.getProperties().get("mavenVersion"));
@@ -84,10 +93,10 @@ public class POMReaderTest extends TestBase {
 
     }
 
+    @Test
     public void testReadHibernateValidatorTckRunner() throws Exception {
-        usePom("hibernate-validator-tck-runner.pom");
         POMReader instance = new POMReader();
-        POMInfo info = instance.readPom(pom);
+        POMInfo info = instance.readPom(tmpDir.usePom("hibernate-validator-tck-runner.pom"));
         assertNotNull(info.getParent());
         assertEquals("org.hibernate", info.getParent().getGroupId());
         assertEquals("hibernate-validator-parent", info.getParent().getArtifactId());
@@ -99,26 +108,26 @@ public class POMReaderTest extends TestBase {
         assertEquals("4.0.2.GA", info.getThisPom().getVersion());
         assertEquals("jar", info.getThisPom().getType());
 
-        assertEquals(6, info.getDependencies().size());
-        Dependency dependency = (Dependency) info.getDependencies().get(5);
+        assertEquals(6, info.getDependencies().get(DEPENDENCIES).size());
+        Dependency dependency = info.getDependencies().get(DEPENDENCIES).get(5);
         assertEquals("org.jboss.test-harness", dependency.getGroupId());
         assertEquals("jboss-test-harness-jboss-as-51", dependency.getArtifactId());
         assertEquals("1.0.0", dependency.getVersion());
         assertEquals("jar", dependency.getType());
 
-        assertEquals(0, info.getDependencyManagement().size());
-        assertEquals(0, info.getProfileDependencies().size());
-        assertEquals(0, info.getProfileDependencyManagement().size());
-        assertEquals(0, info.getPluginManagement().size());
+        assertEquals(0, info.getDependencies().get(DEPENDENCY_MANAGEMENT_LIST).size());
+        assertEquals(0, info.getDependencies().get(PROFILE_DEPENDENCIES).size());
+        assertEquals(0, info.getDependencies().get(PROFILE_DEPENDENCY_MANAGEMENT_LIST).size());
+        assertEquals(0, info.getDependencies().get(PLUGIN_MANAGEMENT).size());
 
-        assertEquals(3, info.getPlugins().size());
-        dependency = (Dependency) info.getPlugins().get(1);
+        assertEquals(3, info.getDependencies().get(PLUGINS).size());
+        dependency = info.getDependencies().get(PLUGINS).get(1);
         assertEquals("org.apache.maven.plugins", dependency.getGroupId());
         assertEquals("maven-surefire-plugin", dependency.getArtifactId());
         assertEquals(null, dependency.getVersion());
         assertEquals("maven-plugin", dependency.getType());
 
-        assertEquals(2, info.getProfilePlugins().size());
+        assertEquals(2, info.getDependencies().get(PROFILE_PLUGINS).size());
 
         assertEquals(3, info.getProperties().size());
         assertEquals("/opt/java/jboss-5.1.0.GA", info.getProperties().get("jboss.home"));
@@ -128,10 +137,10 @@ public class POMReaderTest extends TestBase {
         assertEquals(0, info.getModules().size());
     }
 
+    @Test
     public void testReadHibernateValidator() throws Exception {
-        usePom("hibernate-validator.pom");
         POMReader instance = new POMReader();
-        POMInfo info = instance.readPom(pom);
+        POMInfo info = instance.readPom(tmpDir.usePom("hibernate-validator.pom"));
         assertNotNull(info.getParent());
         assertEquals("org.hibernate", info.getParent().getGroupId());
         assertEquals("hibernate-validator-parent", info.getParent().getArtifactId());
@@ -143,8 +152,8 @@ public class POMReaderTest extends TestBase {
         assertEquals("4.0.2.GA", info.getThisPom().getVersion());
         assertEquals("jar", info.getThisPom().getType());
 
-        assertEquals(6, info.getDependencies().size());
-        Dependency dependency = (Dependency) info.getDependencies().get(0);
+        assertEquals(6, info.getDependencies().get(DEPENDENCIES).size());
+        Dependency dependency = info.getDependencies().get(DEPENDENCIES).get(0);
         assertEquals("javax.validation", dependency.getGroupId());
         assertEquals("validation-api", dependency.getArtifactId());
         assertEquals(null, dependency.getVersion());
@@ -152,7 +161,7 @@ public class POMReaderTest extends TestBase {
         assertEquals("runtime", dependency.getScope());
         assertEquals(false, dependency.isOptional());
 
-        dependency = (Dependency) info.getDependencies().get(1);
+        dependency = info.getDependencies().get(DEPENDENCIES).get(1);
         assertEquals("org.slf4j", dependency.getGroupId());
         assertEquals("slf4j-api", dependency.getArtifactId());
         assertEquals(null, dependency.getVersion());
@@ -160,7 +169,7 @@ public class POMReaderTest extends TestBase {
         assertEquals("runtime", dependency.getScope());
         assertEquals(false, dependency.isOptional());
 
-        dependency = (Dependency) info.getDependencies().get(2);
+        dependency = info.getDependencies().get(DEPENDENCIES).get(2);
         assertEquals("com.googlecode.jtype", dependency.getGroupId());
         assertEquals("jtype", dependency.getArtifactId());
         assertEquals("0.1.0", dependency.getVersion());
@@ -168,7 +177,7 @@ public class POMReaderTest extends TestBase {
         assertEquals("runtime", dependency.getScope());
         assertEquals(false, dependency.isOptional());
 
-        dependency = (Dependency) info.getDependencies().get(3);
+        dependency = info.getDependencies().get(DEPENDENCIES).get(3);
         assertEquals("org.slf4j", dependency.getGroupId());
         assertEquals("slf4j-log4j12", dependency.getArtifactId());
         assertEquals(null, dependency.getVersion());
@@ -176,7 +185,7 @@ public class POMReaderTest extends TestBase {
         assertEquals("runtime", dependency.getScope());
         assertEquals(true, dependency.isOptional());
 
-        dependency = (Dependency) info.getDependencies().get(4);
+        dependency = info.getDependencies().get(DEPENDENCIES).get(4);
         assertEquals("org.hibernate.java-persistence", dependency.getGroupId());
         assertEquals("jpa-api", dependency.getArtifactId());
         assertEquals("2.0.Beta-20090815", dependency.getVersion());
@@ -184,7 +193,7 @@ public class POMReaderTest extends TestBase {
         assertEquals("runtime", dependency.getScope());
         assertEquals(true, dependency.isOptional());
 
-        dependency = (Dependency) info.getDependencies().get(5);
+        dependency = info.getDependencies().get(DEPENDENCIES).get(5);
         assertEquals("org.testng", dependency.getGroupId());
         assertEquals("testng", dependency.getArtifactId());
         assertEquals(null, dependency.getVersion());
@@ -193,76 +202,76 @@ public class POMReaderTest extends TestBase {
         assertEquals(false, dependency.isOptional());
         assertEquals("jdk15", dependency.getClassifier());
 
-        assertEquals(0, info.getDependencyManagement().size());
-        assertEquals(2, info.getProfileDependencies().size());
-        assertEquals(0, info.getProfileDependencyManagement().size());
-        assertEquals(0, info.getPluginManagement().size());
+        assertEquals(0, info.getDependencies().get(DEPENDENCY_MANAGEMENT_LIST).size());
+        assertEquals(2, info.getDependencies().get(PROFILE_DEPENDENCIES).size());
+        assertEquals(0, info.getDependencies().get(PROFILE_DEPENDENCY_MANAGEMENT_LIST).size());
+        assertEquals(0, info.getDependencies().get(PLUGIN_MANAGEMENT).size());
 
-        assertEquals(1, info.getReportingPlugins().size());
-        dependency = (Dependency) info.getReportingPlugins().get(0);
+        assertEquals(1, info.getDependencies().get(REPORTING_PLUGINS).size());
+        dependency = info.getDependencies().get(REPORTING_PLUGINS).get(0);
         assertEquals("org.apache.maven.plugins", dependency.getGroupId());
         assertEquals("maven-project-info-reports-plugin", dependency.getArtifactId());
         assertEquals("2.0.1", dependency.getVersion());
         assertEquals("maven-plugin", dependency.getType());
 
-        assertEquals(9, info.getPlugins().size());
-        dependency = (Dependency) info.getPlugins().get(0);
+        assertEquals(9, info.getDependencies().get(PLUGINS).size());
+        dependency = info.getDependencies().get(PLUGINS).get(0);
         assertEquals("org.apache.maven.plugins", dependency.getGroupId());
         assertEquals("maven-source-plugin", dependency.getArtifactId());
         assertEquals(null, dependency.getVersion());
         assertEquals("maven-plugin", dependency.getType());
 
-        dependency = (Dependency) info.getPlugins().get(1);
+        dependency = info.getDependencies().get(PLUGINS).get(1);
         assertEquals("org.codehaus.mojo", dependency.getGroupId());
         assertEquals("jaxb2-maven-plugin", dependency.getArtifactId());
         assertEquals(null, dependency.getVersion());
         assertEquals("maven-plugin", dependency.getType());
 
-        dependency = (Dependency) info.getPlugins().get(2);
+        dependency = info.getDependencies().get(PLUGINS).get(2);
         assertEquals("org.apache.maven.plugins", dependency.getGroupId());
         assertEquals("maven-surefire-plugin", dependency.getArtifactId());
         assertEquals(null, dependency.getVersion());
         assertEquals("maven-plugin", dependency.getType());
 
-        dependency = (Dependency) info.getPlugins().get(3);
+        dependency = info.getDependencies().get(PLUGINS).get(3);
         assertEquals("org.apache.maven.plugins", dependency.getGroupId());
         assertEquals("maven-surefire-report-plugin", dependency.getArtifactId());
         assertEquals(null, dependency.getVersion());
         assertEquals("maven-plugin", dependency.getType());
 
-        dependency = (Dependency) info.getPlugins().get(4);
+        dependency = info.getDependencies().get(PLUGINS).get(4);
         assertEquals("org.apache.maven.plugins", dependency.getGroupId());
         assertEquals("maven-shade-plugin", dependency.getArtifactId());
         assertEquals(null, dependency.getVersion());
         assertEquals("maven-plugin", dependency.getType());
 
-        dependency = (Dependency) info.getPlugins().get(5);
+        dependency = info.getDependencies().get(PLUGINS).get(5);
         assertEquals("org.jboss.maven.plugins", dependency.getGroupId());
         assertEquals("maven-jdocbook-plugin", dependency.getArtifactId());
         assertEquals("2.2.0", dependency.getVersion());
         assertEquals("maven-plugin", dependency.getType());
 
-        dependency = (Dependency) info.getPlugins().get(6);
+        dependency = info.getDependencies().get(PLUGINS).get(6);
         assertEquals("org.jboss.maven.plugins", dependency.getGroupId());
         assertEquals("maven-jdocbook-style-plugin", dependency.getArtifactId());
         assertEquals("2.0.0", dependency.getVersion());
         assertEquals("maven-plugin", dependency.getType());
 
-        dependency = (Dependency) info.getPlugins().get(7);
+        dependency = info.getDependencies().get(PLUGINS).get(7);
         assertEquals("org.apache.maven.plugins", dependency.getGroupId());
         assertEquals("maven-assembly-plugin", dependency.getArtifactId());
         assertEquals(null, dependency.getVersion());
         assertEquals("maven-plugin", dependency.getType());
 
-        dependency = (Dependency) info.getPlugins().get(8);
+        dependency = info.getDependencies().get(PLUGINS).get(8);
         assertEquals("org.twdata.maven", dependency.getGroupId());
         assertEquals("maven-cli-plugin", dependency.getArtifactId());
         assertEquals("0.6.3.CR3", dependency.getVersion());
         assertEquals("maven-plugin", dependency.getType());
 
-        assertEquals(0, info.getProfilePlugins().size());
-        assertEquals(1, info.getPluginDependencies().size());
-        dependency = (Dependency) info.getPluginDependencies().get(0);
+        assertEquals(0, info.getDependencies().get(PROFILE_PLUGINS).size());
+        assertEquals(1, info.getDependencies().get(PLUGIN_DEPENDENCIES).size());
+        dependency = info.getDependencies().get(PLUGIN_DEPENDENCIES).get(0);
         assertEquals("org.hibernate", dependency.getGroupId());
         assertEquals("hibernate-jdocbook-style", dependency.getArtifactId());
         assertEquals("2.0.0", dependency.getVersion());
@@ -272,10 +281,10 @@ public class POMReaderTest extends TestBase {
         assertEquals(0, info.getModules().size());
     }
 
+    @Test
     public void testReadMojoParent() throws Exception {
-        usePom("mojo-parent.pom");
         POMReader instance = new POMReader();
-        POMInfo info = instance.readPom(pom);
+        POMInfo info = instance.readPom(tmpDir.usePom("mojo-parent.pom"));
         assertNotNull(info.getParent());
         assertEquals("org.codehaus", info.getParent().getGroupId());
         assertEquals("codehaus-parent", info.getParent().getArtifactId());
@@ -287,54 +296,54 @@ public class POMReaderTest extends TestBase {
         assertEquals("27", info.getThisPom().getVersion());
         assertEquals("pom", info.getThisPom().getType());
 
-        assertEquals(0, info.getDependencies().size());
-        assertEquals(2, info.getDependencyManagement().size());
-        Dependency dependency = (Dependency) info.getDependencyManagement().get(0);
+        assertEquals(0, info.getDependencies().get(DEPENDENCIES).size());
+        assertEquals(2, info.getDependencies().get(DEPENDENCY_MANAGEMENT_LIST).size());
+        Dependency dependency = info.getDependencies().get(DEPENDENCY_MANAGEMENT_LIST).get(0);
         assertEquals("org.apache.maven", dependency.getGroupId());
         assertEquals("maven-plugin-api", dependency.getArtifactId());
         assertEquals("2.0", dependency.getVersion());
         assertEquals("jar", dependency.getType());
 
-        assertEquals(1, info.getExtensions().size());
+        assertEquals(1, info.getDependencies().get(EXTENSIONS).size());
 
-        assertEquals(24, info.getPluginManagement().size());
-        dependency = (Dependency) info.getPluginManagement().get(17);
+        assertEquals(24, info.getDependencies().get(PLUGIN_MANAGEMENT).size());
+        dependency = info.getDependencies().get(PLUGIN_MANAGEMENT).get(17);
         assertEquals("org.apache.maven.plugins", dependency.getGroupId());
         assertEquals("maven-site-plugin", dependency.getArtifactId());
         assertEquals("2.1.1", dependency.getVersion());
         assertEquals("maven-plugin", dependency.getType());
-        dependency = (Dependency) info.getPluginManagement().get(18);
+        dependency = info.getDependencies().get(PLUGIN_MANAGEMENT).get(18);
         assertEquals("org.apache.maven.plugins", dependency.getGroupId());
         assertEquals("maven-source-plugin", dependency.getArtifactId());
         assertEquals("2.1.2", dependency.getVersion());
         assertEquals("maven-plugin", dependency.getType());
 
-        assertEquals(1, info.getPluginManagementDependencies().size());
-        dependency = (Dependency) info.getPluginManagementDependencies().get(0);
+        assertEquals(1, info.getDependencies().get(PLUGIN_MANAGEMENT_DEPENDENCIES).size());
+        dependency = info.getDependencies().get(PLUGIN_MANAGEMENT_DEPENDENCIES).get(0);
         assertEquals("org.apache.maven.doxia", dependency.getGroupId());
         assertEquals("doxia-module-xhtml", dependency.getArtifactId());
         assertEquals("1.1.3", dependency.getVersion());
         assertEquals("jar", dependency.getType());
 
-        assertEquals(2, info.getPlugins().size());
-        dependency = (Dependency) info.getPlugins().get(1);
+        assertEquals(2, info.getDependencies().get(PLUGINS).size());
+        dependency = info.getDependencies().get(PLUGINS).get(1);
         assertEquals("org.codehaus.mojo", dependency.getGroupId());
         assertEquals("cobertura-maven-plugin", dependency.getArtifactId());
         assertEquals(null, dependency.getVersion());
         assertEquals("maven-plugin", dependency.getType());
 
-        assertEquals(2, info.getReportingPlugins().size());
-        dependency = (Dependency) info.getReportingPlugins().get(1);
+        assertEquals(2, info.getDependencies().get(REPORTING_PLUGINS).size());
+        dependency = info.getDependencies().get(REPORTING_PLUGINS).get(1);
         assertEquals("org.apache.maven.plugins", dependency.getGroupId());
         assertEquals("maven-project-info-reports-plugin", dependency.getArtifactId());
         assertEquals("2.2", dependency.getVersion());
         assertEquals("maven-plugin", dependency.getType());
 
-        assertEquals(6, info.getProfilePlugins().size());
-        assertEquals(8, info.getProfileReportingPlugins().size());
-        assertEquals(1, info.getProfilePluginManagement().size());
-        assertEquals(0, info.getProfileDependencies().size());
-        assertEquals(0, info.getProfileDependencyManagement().size());
+        assertEquals(6, info.getDependencies().get(PROFILE_PLUGINS).size());
+        assertEquals(8, info.getDependencies().get(PROFILE_REPORTING_PLUGINS).size());
+        assertEquals(1, info.getDependencies().get(PROFILE_PLUGIN_MANAGEMENT).size());
+        assertEquals(0, info.getDependencies().get(PROFILE_DEPENDENCIES).size());
+        assertEquals(0, info.getDependencies().get(PROFILE_DEPENDENCY_MANAGEMENT_LIST).size());
 
         assertEquals(5, info.getProperties().size());
         assertEquals("UTF-8", info.getProperties().get("project.build.sourceEncoding"));
@@ -346,4 +355,53 @@ public class POMReaderTest extends TestBase {
         assertEquals(0, info.getModules().size());
     }
 
-}
\ No newline at end of file
+    @Test
+    public void testReadClojurePom() throws Exception {
+        POMReader instance = new POMReader();
+        POMInfo info = instance.readPom(tmpDir.usePom("repository/org/clojure/clojure/1.2.1/clojure-1.2.1.pom"));
+        assertNotNull("null pom", info.getThisPom());
+        assertEquals("groupId", "org.clojure", info.getThisPom().getGroupId());
+        assertEquals("artifactId", "clojure", info.getThisPom().getArtifactId());
+        assertEquals("version", "1.2.1", info.getThisPom().getVersion());
+        assertEquals("type", "jar", info.getThisPom().getType());
+        assertEquals("debian.mavenRules", "* clojure * s/([0-9]+)\\.([0-9]+).*/$1.$2.x/ * *", info.getProperties().get("debian.mavenRules"));
+    }
+
+    @Test
+    public void testReadBndPom() throws Exception {
+        POMReader instance = new POMReader();
+        POMInfo info = instance.readPom(tmpDir.usePom("bndlib.pom"));
+        assertNotNull("null pom", info.getThisPom());
+        assertEquals("groupId", "biz.aQute", info.getThisPom().getGroupId());
+        assertEquals("artifactId", "bndlib", info.getThisPom().getArtifactId());
+        assertEquals("version", "1.43.0", info.getThisPom().getVersion());
+    }
+
+    @Test
+    public void testTreePathMatches() {
+        TreePath<String> path = new TreePath<String>();
+        path.add("a");
+        path.add("b");
+        path.add("c");
+        path.add("d");
+
+        assertTrue(path.matches("a/b/c/d"));
+        assertTrue(path.matches("b/c/d"));
+        assertTrue(path.matches("c/d"));
+        assertTrue(path.matches("d"));
+
+        assertTrue(path.matches("*/b/c/d"));
+        assertTrue(path.matches("a/*/c/d"));
+        assertTrue(path.matches("a/b/*/d"));
+        assertTrue(path.matches("a/b/c/*"));
+
+        assertTrue(path.matches("/*/b/c/d"));
+        assertTrue(path.matches("/a/*/c/d"));
+        assertTrue(path.matches("/a/b/*/d"));
+        assertTrue(path.matches("/a/b/c/*"));
+
+        assertFalse(path.matches("/b/c/d"));
+        assertFalse(path.matches("/c/d"));
+        assertFalse(path.matches("/d"));
+    }
+}
